-- miscellaneous functions extracted from Main so
-- we can more easily test them

module Lib exposing(looseLabels)


maybe default val =
    case val of
        Just v -> v
        Nothing -> default

-- https://github.com/cenfun/nice-ticks/blob/master/docs/Nice-Numbers-for-Graph-Labels.pdf

log10 x = logBase 10 x
expt b x = b^(toFloat x)


niceDigitRound f =
    let digits = [1, 2, 5, 10]
        breaks = List.map2 (\ a b -> (a+b)/2)  (List.drop 1 digits) digits
        m = List.map2 Tuple.pair digits breaks
    in case List.filter (\ d -> (f  < Tuple.second d)) m  of
           ((a,_) :: _) -> a
           _ -> 10

niceDigit f =
    let niceDigits = [1, 2, 5]
    in case List.filter ((<) f) niceDigits of
           (a :: _) -> a
           _ -> 10

niceNumber x round =
    let exp = floor (log10 x)
        f = x / (expt 10.0 exp)
    in
        if round
        then
            (niceDigitRound f) * expt 10 exp
        else
            (niceDigit f) * expt 10 exp

looseLabels ticks min max =
    let
        range = niceNumber (max-min) False
        d = niceNumber (range/(ticks - 1)) True
        graphmin = toFloat (floor  (min/d)) * d
        graphmax = toFloat (ceiling (max/d)) * d
    in (graphmin, graphmax, d)
